require "shared.lua"

camera = {}
camera._x = 0
camera._y = 0
camera.scaleX = 1
camera.scaleY = 1
camera.rotation = 0
camera.zoomchanged = false
camera.layers = {}
camera.zoomAnimationState = ZOOMED
camera.zoomTimer = 0
camera.zoomSpeed = 1
camera.defaultZoomIn = 4
camera.defaultZoomOut = 0.5
camera.goalZoom = 1;
camera.currentZoom = 1

-- Camera animation - START

function camera:zoomout()
	if currentZoom ~= self.defaultZoomOut then
		self.goalZoom = self.defaultZoomOut;
		self.zoomAnimationState = ZOOMING
	end
end

function camera:zoomin()
	if currentZoom ~= self.defaultZoomOut then
		self.goalZoom = self.defaultZoomIn;
		self.zoomAnimationState = ZOOMING
	end
end

function camera:zoom(goal)
	if currentZoom ~= goal then
		self.goalZoom = goal;
		self.zoomAnimationState = ZOOMING
	end
end

function camera:updateZoom(dt)
	
	if self.zoomAnimationState == ZOOMED then 
		return false
	end
	
	if self.zoomAnimationState == ZOOMING then
	
		if self.currentZoom < self.goalZoom then
			
			self.currentZoom = self.currentZoom + self.zoomSpeed*dt
		
			if self.currentZoom >= self.goalZoom then
				self.zoomAnimationState = ZOOMED 
				self.currentZoom = self.goalZoom
				if self.delegate then self.delegate.zoomFinished(self.delegate) end
			end
			
		else 
			self.currentZoom = self.currentZoom - self.zoomSpeed*dt
		
			if self.currentZoom <= self.goalZoom then
				self.zoomAnimationState = ZOOMED 
				self.currentZoom = self.goalZoom
				if self.delegate then self.delegate.zoomFinished(self.delegate) end
			end
		
		end
	
	end
	
	self.scaleX = self.currentZoom
	self.scaleY = self.currentZoom
	
	return true
	
end

function camera:update(dt)
	camera.zoomchanged = self:updateZoom(dt)
end


-- Camera animation - END

function camera:newLayer(scale, func)
  table.insert(self.layers, { draw = func, scale = scale })
  table.sort(self.layers, function(a, b) return a.scale < b.scale end)
end

function camera:newLayerClass(layer)
  table.insert(self.layers, layer)
  table.sort(self.layers, function(a, b) return a.scale < b.scale end)
end

function camera:set()
  love.graphics.push()
  love.graphics.rotate(-self.rotation)
  love.graphics.scale(self.scaleX,self.scaleY)
  love.graphics.translate(-self._x, -self._y)
end

function camera:unset()
  love.graphics.pop()
end

function camera:move(dx, dy)
  self._x = self._x + (dx or 0)
  self._y = self._y + (dy or 0)
end

function camera:rotate(dr)
  self.rotation = self.rotation + dr
end

function camera:scale(sx, sy)
  camera.currentZoom = sx or 1
  sx = sx or 1
  self.scaleX = sx--self.scaleX * sx
  self.scaleY = sy or sx--self.scaleY * (sy or sx)
  self.zoomchanged = true
end

function camera:setX(value)
  if self._bounds then
    self._x = math.clamp(value, self._bounds.x1, self._bounds.x2)
  else
    self._x = value
  end
end

function camera:setY(value)
    if self._bounds then
		self._y = math.clamp(value, self._bounds.y1, self._bounds.y2)
    else
		self._y = value
	end
end

function camera:setPosition(x, y)
	if x then self:setX(x) end
	if y then self:setY(y) end
end

function camera:setScale(sx, sy)
	self.scaleX = sx or self.scaleX
	self.scaleY = sy or self.scaleY
end

function camera:getBounds()
	return unpack(self._bounds)
end

function camera:setBounds(x1, y1, x2, y2)
	self._bounds = { x1 = x1, y1 = y1, x2 = x2, y2 = y2 }
end

function camera:mousePosition()
	return love.mouse.getX() * self.scaleX + self._x, love.mouse.getY() * self.scaleY + self._y
end

function camera:mouseRealPosition()
	return love.mouse.getX() / self.scaleX + self._x, love.mouse.getY() / self.scaleY + self._y
end

function camera:transformPosition(x,y)
	return x * self.scaleX + self._x, y * self.scaleY + self._y
end

function camera:draw()
  local bx, by = self._x, self._y
  
  for _, v in ipairs(self.layers) do
    self._x = bx * v.scale
    self._y = by * v.scale
    camera:set()
    v.draw()
    camera:unset()
  end
  
end
