require "actor.lua"
require "controls.lua" 
require "defines.lua" 
require "sprite.lua" 
require "circle.lua"
require "circleorbs.lua"
require "hero.lua"
require "light.lua"

-- Enemy type A
EnemyA = Actor:new()

function EnemyA:init(_x,_y,_level)
	
	self.level = _level or 1
	self.spriteW = 112
	self.spriteH = 118
	
	if self.level == 10 then
		self.spriteScale =  2.0
	else
		self.spriteScale =  0.5 +(self.level*0.05)
	end
	
	self.type = "A"
	
	Actor.initBox(self,_x,_y,self.spriteW*self.spriteScale,self.spriteH*self.spriteScale)
	
	self.speed = 30 + (8*self.level)
	self.friction = 100
	self:reset()
	
	--Sprite 128x128
	local image = love.graphics.newImage("gfx/enemy_a.png")
	local quads = {}
	quads[1] = love.graphics.newQuad(0,0,self.spriteW,self.spriteH,128,128)
	
	self.sprite = Sprite:new()
	self.sprite:setSize(self.spriteW,self.spriteH,self.w,self.h)
	self.sprite:init(image,quads,1)
	
	local sprite = Sprite:new()
	sprite:init(image,quads,1)

	
	--Activate / deactivate
	self.lightAnimationState = LIGHT_OFF
	self.lightTimer = 0
	self.lightSpeed = 255
	self.defaultLightOn = 255
	self.defaultLightOff = 0
	self.defaultLight = 0
	self.rotation = 0
	
end

function EnemyA:deactivate()
	if self.lightAnimationState ~= LIGHT_OFF then
		self.lightAnimationState = LIGHT_TURNING_OFF
	end
end

function EnemyA:activate()
	self.lightAnimationState = LIGHT_TURNING_ON
end

function EnemyA:updateLight(dt)
	
	if self.lightAnimationState == LIGHT_OFF or self.lightAnimationState == LIGHT_ON then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
		end
		
	end
	
end


function EnemyA:update(dt, hero)
	
	if self.lightAnimationState == LIGHT_OFF then 
		if (hero.x > self.x) then
			self.rotation = math.atan((self.y - hero.y)/(self.x - hero.x)) + math.pi
		else
			self.rotation = math.atan((self.y - hero.y)/(self.x - hero.x))
		end
		return 
	end
	-- update position
	if (hero.x > self.x) then
		self.rotation = math.atan((self.y - hero.y)/(self.x - hero.x)) + math.pi
		self.x = self.x + dt*self.speed
	else
		self.rotation = math.atan((self.y - hero.y)/(self.x - hero.x))
		self.x = self.x - dt*self.speed
	end
	
	if (hero.y > self.y) then
		self.y = self.y + dt*self.speed
	else
		self.y = self.y - dt*self.speed
	end
	
	--Check pressed keys
	Actor.update(self,dt)	
	self:updateLight(dt)
	
end

function EnemyA:draw()
	
	--love.graphics.setColor(self.defaultLight,self.defaultLight,self.defaultLight,255)
	love.graphics.setColor(0,0,0,255)
	self.sprite:draw(self.x + self.w/2,self.y + self.h/2,self.rotation,self.spriteW/2,self.spriteH/2)

	if SHOW_DEBUG_BOXES then
		self:debugDraw()
	end

end