require "actor.lua"
require "controls.lua" 
require "defines.lua" 
require "sprite.lua" 
require "circle.lua"
require "circleorbs.lua"
require "hero.lua"
require "light.lua"

-- Enemy type A
EnemyB = Actor:new()

function EnemyB:init(_x,_y,_level)
	
	self.level = _level or 1
	self.spriteW = 300
	self.spriteH = 146
	self.spriteScale =  0.5 +(self.level*0.05)
	self.type = "B"
	
	Actor.initBox(self,_x,_y,self.spriteW*self.spriteScale,self.spriteH*self.spriteScale)
	
	self.speed = 90 + (10*self.level)
	self.friction = 100
	self:reset()
	
	--Sprite 128x128
	local image = love.graphics.newImage("gfx/enemy_b_active.png")
	local quads = {}
	quads[1] = love.graphics.newQuad(0,0,self.spriteW,self.spriteH,512,256)
	
	self.sprite = Sprite:new()
	self.sprite:setSize(self.spriteW,self.spriteH,self.w,self.h)
	self.sprite:init(image,quads,1)
	
	image = love.graphics.newImage("gfx/enemy_b_inactive.png")
	quads = {}
	quads[1] = love.graphics.newQuad(0,0,self.spriteW,self.spriteH,512,256)
	
	self.sprite_inactive = Sprite:new()
	self.sprite_inactive:setSize(self.spriteW,self.spriteH,self.w,self.h)
	self.sprite_inactive:init(image,quads,1)
	self.awake = false

	
	--Activate / deactivate
	self.lightAnimationState = LIGHT_OFF
	self.lightTimer = 0
	self.lightSpeed = 255
	self.defaultLightOn = 255
	self.defaultLightOff = 0
	self.defaultLight = 0
	self.rotation = 0
	
	-- UGLY FAST HACK
	self.timer = 0
	self.offsetX = 0
	self.offsetY = 0
	self.oldOffsetX = 0
	self.oldOffsetY = 0
	
end

function EnemyB:deactivate()
	if self.lightAnimationState ~= LIGHT_OFF then
		self.lightAnimationState = LIGHT_TURNING_OFF
	end
end

function EnemyB:activate()
	self.lightAnimationState = LIGHT_TURNING_ON
end

function EnemyB:updateLight(dt)
	
	if self.lightAnimationState == LIGHT_OFF or self.lightAnimationState == LIGHT_ON then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
		end
		
	end
	
end


function EnemyB:update(dt, hero)
	
	if self.lightAnimationState == LIGHT_OFF then return end
	
	if self.awake then
		if (hero.x > self.x) then
			self.x = self.x + dt*self.speed
		else
			self.x = self.x - dt*self.speed
		end
		
		if (hero.y > self.y) then
			self.y = self.y + dt*self.speed
		else
			self.y = self.y - dt*self.speed
		end
	else
		self.timer = self.timer+dt*0.5
		if self.level == 1 then
			self.oldOffsetX = self.offsetX
			self.offsetX = math.sin(self.timer)
			self.x = self.x + (self.oldOffsetX-self.offsetX)*self.speed*2
		elseif self.level == 2 then
			self.oldOffsetY = self.offsetY
			self.offsetY = math.sin(self.timer)
			self.y = self.y + (self.oldOffsetY-self.offsetY)*self.speed*2
		elseif self.level == 3 then
			self.oldOffsetX = self.offsetX
			self.offsetX = math.sin(self.timer)
			self.x = self.x + (self.oldOffsetX-self.offsetX)*self.speed*2
			self.oldOffsetY = self.offsetY
			self.offsetY = math.cos(self.timer)
			self.y = self.y + (self.oldOffsetY-self.offsetY)*self.speed*2
		end
	end
	
	self:updateLight(dt)
	
end

function EnemyB:draw()
	
	love.graphics.setColor(self.defaultLight,self.defaultLight,self.defaultLight,255)
	
	if self.awake then
		self.sprite:draw(self.x + self.w/2,self.y + self.h/2,self.rotation,self.spriteW/2,self.spriteH/2)
	else
		self.sprite_inactive:draw(self.x + self.w/2,self.y + self.h/2,self.rotation,self.spriteW/2,self.spriteH/2)
	end
	
	if SHOW_DEBUG_BOXES then
		self:debugDraw()
	end

end