require "actor.lua"
require "controls.lua" 
require "defines.lua" 
require "sprite.lua" 
require "circle.lua"
require "circleorbs.lua" 

-- Hero
Hero = Actor:new()

function Hero:init(_x,_y)
	Actor.init(self,_x,_y,50)
	self.speed = 100
	self.friction = 100
	self:reset()
	
	--Sprite 128x128
	local image = love.graphics.newImage("gfx/player.png")
	local quads = {}
	quads[1] = love.graphics.newQuad(0,0,100,100,128,128)
	
	self.sprite = Sprite:new()
	self.sprite:init(image,quads,1)
	--sprite:setSize(100,100,50,50)
	
	--Sprite 128x128
	image = love.graphics.newImage("gfx/player_orb.png")
	quads = {}
	quads[1] = love.graphics.newQuad(0,0,20,20,32,32)
	
	local sprite = Sprite:new()
	sprite:init(image,quads,1)
	
	self.circleorbs = CircleOrbs:new()
	self.circleorbs:init(0,70,5,sprite)
	
	self.controllable = true
	
	--Activate / deactivate
	self.lightAnimationState = LIGHT_ON
	self.lightTimer = 0
	self.lightSpeed = 255
	self.defaultLightOn = 255
	self.defaultLightOff = 150
	self.defaultLight = 255
	
end

function Hero:deactivate()
	if self.lightAnimationState ~= LIGHT_OFF then
		self.lightAnimationState = LIGHT_TURNING_OFF
	end
end

function Hero:activate()
	self.lightAnimationState = LIGHT_TURNING_ON
end

function Hero:updateLight(dt)
	
	if self.lightAnimationState == LIGHT_OFF or self.lightAnimationState == LIGHT_ON then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
		end
		
	end
	
end

function Hero:update(dt, controls)
	
	if self.controllable then
	
		if controls.up then
			self.accY = -self.speed
		elseif controls.down then
			self.accY = self.speed
		else 
			self.accY = 0
		end
		
		if controls.left then
			self.accX = -self.speed
		elseif controls.right then
			self.accX = self.speed
		else 
			self.accX = 0
		end
	
	end
	
	--Check pressed keys
	Actor.update(self,dt)
	
	--Update orbs
	self.circleorbs:update(dt)
	
	self:updateLight(dt)
	
end

function Hero:draw()
	
	love.graphics.setColor(255-self.defaultLight,255-self.defaultLight,255-self.defaultLight,255)
	self.sprite:draw(self.x,self.y)
	self.circleorbs:draw(self.x + self.w/2,self.y + self.h/2,20,20)
	
	if SHOW_DEBUG_BOXES then
		self:debugDraw()
	end

end