require "actor.lua"
require "controls.lua" 
require "defines.lua" 
require "sprite.lua" 
require "circle.lua"
require "circleorbs.lua" 

-- Indicator
Indicator = Actor:new()

function Indicator:init(_number,_x,_y)

	Actor.init(self,_x-25,_y-25,25)
	
	self.number = _number

	--Sprite 50x50
	local image = love.graphics.newImage("gfx/indicator.png")
	local quads = {}
	quads[1] = love.graphics.newQuad(0,0,50,50,64,64)
	
	self.sprite = Sprite:new()
	self.sprite:init(image,quads,1)
	--sprite:setSize(100,100,50,50)
	
	--Sprite 128x128
	image = love.graphics.newImage("gfx/indicator_orb.png")
	quads = {}
	quads[1] = love.graphics.newQuad(0,0,15,15,16,16)
	
	local sprite = Sprite:new()
	sprite:init(image,quads,1)
	
	self.circleorbs = CircleOrbs:new()
	self.circleorbs:init(self.number,40,5,sprite)
	
	--Activate / deactivate
	self.lightAnimationState = LIGHT_OFF
	self.lightTimer = 0
	self.lightSpeed = INDICATOR_SHOW_SPEED
	self.defaultLightOn = 255
	self.defaultLightOff = 0
	self.defaultLight = 0
	self.timer = 0
	self.wait = 1
	self.isDead = false
	
	self.font = love.graphics.newImageFont("gfx/font.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`'*#=[]\"")
	self.showThanks = false
	
end

function Indicator:setNumberTo(_number)
	self.number = _number
	self.circleorbs:setNumber(_number)
end

function Indicator:deactivate()
	self.lightAnimationState = LIGHT_TURNING_OFF
end

function Indicator:die()
	self:deactivate()
	self.isDead = true
end

function Indicator:start()
	self.lightAnimationState = LIGHT_TURNING_ON
end

function Indicator:updateLight(dt)
	
	if self.lightAnimationState == LIGHT_OFF then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
			if self.delegate and self.showThanks == false then self.delegate.indicatorFinished(self.delegate) end
		end
	
	elseif self.lightAnimationState == LIGHT_ON then
		self.timer = self.timer + dt
		if self.timer > self.wait and self.showThanks == false then
			self.lightAnimationState = LIGHT_TURNING_OFF
		end
	end
	
end

function Indicator:update(dt)

	--Update light	
	self:updateLight(dt)
	
	if self.isDead then return end
	
	--Update orbs
	self.circleorbs:update(dt)
	
	
end

function Indicator:draw()
	
	if self.defaultLight == 0 then return end
	
	love.graphics.setColor(255,255,255,self.defaultLight)
	
	if self.showThanks then
		love.graphics.setColor(255,255,255,self.defaultLight)
		love.graphics.setFont(self.font);
		love.graphics.print("void end",30,460)
		love.graphics.print("thank you for playing",30,500)
		love.graphics.print("by diux",30,540)
	end
	
	self.sprite:draw(self.x,self.y)
	self.circleorbs:draw(self.x + self.w/2,self.y + self.h/2,15,15)
	
	if SHOW_DEBUG_BOXES then
		self:debugDraw()
	end

end