require "actor.lua"
require "controls.lua" 
require "defines.lua" 
require "sprite.lua" 
require "circle.lua"
require "circleorbs.lua" 

-- Item
Item = Actor:new()

function Item:init(_number,_x,_y)

	Actor.init(self,_x,_y,25)
	
	self.number = _number

	--Sprite 50x50
	local image = love.graphics.newImage("gfx/item.png")
	local quads = {}
	quads[1] = love.graphics.newQuad(0,0,50,50,64,64)
	
	self.sprite = Sprite:new()
	self.sprite:init(image,quads,1)
	--sprite:setSize(100,100,50,50)
	
	--Sprite 128x128
	image = love.graphics.newImage("gfx/item_orb.png")
	quads = {}
	quads[1] = love.graphics.newQuad(0,0,15,15,16,16)
	
	local sprite = Sprite:new()
	sprite:init(image,quads,1)
	
	self.circleorbs = CircleOrbs:new()
	self.circleorbs:init(self.number,40,5,sprite)
	
	--Activate / deactivate
	self.lightAnimationState = LIGHT_ON
	self.lightTimer = 0
	self.lightSpeed = 255
	self.defaultLightOn = 255
	self.defaultLightOff = 0
	self.defaultLight = 255
	self.isDead = false
	
end

function Item:deactivate()
	if self.lightAnimationState ~= LIGHT_OFF then
		self.lightAnimationState = LIGHT_TURNING_OFF
	end
end

function Item:die()
	self:deactivate()
	self.isDead = true
end

function Item:activate()
	if self.isDead then return end
	self.lightAnimationState = LIGHT_TURNING_ON
end

function Item:updateLight(dt)
	
	if self.lightAnimationState == LIGHT_OFF or self.lightAnimationState == LIGHT_ON then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
		end
		
	end
	
end

function Item:update(dt)

	--Update light	
	self:updateLight(dt)
	
	if self.isDead then return end
	
	--Update orbs
	self.circleorbs:update(dt)
	
	
end

function Item:draw()
	
	love.graphics.setColor(self.defaultLight,self.defaultLight,self.defaultLight,255)
	self.sprite:draw(self.x,self.y)
	self.circleorbs:draw(self.x + self.w/2,self.y + self.h/2,15,15)
	
	if SHOW_DEBUG_BOXES then
		self:debugDraw()
	end

end