require "circle.lua"

-- Light
Light = Circle:new()

function Light:init(_x,_y,_r)
	Circle.init(self,_x,_y,_r)
	self:setDetails()
	self.time = 0
	self.lightR = _r
	
		--Activate / deactivate
	self.lightAnimationState = LIGHT_ON
	self.lightTimer = 0
	self.lightSpeed = 1
	self.defaultLightOn = 1
	self.defaultLightOff = 0
	self.defaultLight = 1
	self.isDead = false
end

function Light:setDetails(_intensity,_speed,_offset)
	self.intensityR = _intensityR or 100
	self.intensityG = _intensityG or 100
	self.intensityB = _intensityB or 100
	self.speed = _speed or 5
	self.offset = _roffset or 10
end

function Light:deactivate()
	if self.lightAnimationState ~= LIGHT_OFF then
		self.lightAnimationState = LIGHT_TURNING_OFF
	end
end

function Light:die()
	self:deactivate()
	self.isDead = true
end

function Light:activate()
	if self.isDead then return end
	self.lightAnimationState = LIGHT_TURNING_ON
end

function Light:updateLight(dt)
	
	if self.lightAnimationState == LIGHT_OFF or self.lightAnimationState == LIGHT_ON then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
		end
		
	end
	
end

function Light:update(dt)
	self:updateLight(dt)
	self.time = self.time + dt*self.speed 
	self.lightR = (self.r + math.sin(self.time)*self.offset)*self.defaultLight
end

function Light:debugDraw()
	love.graphics.setColor(unpack(self.color))
	love.graphics.circle('line',self.x,self.y,self.r,20)
end
