require "defines.lua"
require "shared.lua"
require "camera.lua"
require "box.lua"
require "actor.lua"
require "sprite.lua"
require "stagemanager.lua"
require "hero.lua" 
require "controls.lua" 
require "stage1.lua" 

math.randomseed(os.time())
math.random()
math.random()
math.random()

isFullscreen = false

function love.load(args)
	
	love.mouse.setVisible(false);

	local hero = Hero:new()
	hero:init(300,300)

	controls = Controls:new()
	camera:newLayer(1,drawLayer)	
	
	stage = StageManager:new()
	stage:init("stage1",hero,camera)
	camera:scale(STAGE_TRANSITION_ZOOM)
	love.graphics.setCaption("void")
	
end

function love.update(dt)
	camera:update(dt)
	stage:update(dt,controls)
end

function drawLayer()
	stage:draw()
end

function love.draw()
    --love.graphics.setBackgroundColor( 255, 255, 255 )
	camera:draw()
	stage:offCameraDraw()
	
  if SHOW_FPS then
	love.graphics.print("FPS: " .. love.timer.getFPS(), 2, 2)
  end
   
end

function love.keypressed(key, unicode)
	
	if key == "up" or key == "w" then
		controls.up = true
	end
	
	if key == "down" or key == "s"  then
		controls.down = true
	end
	
	if key == "left" or key == "a"  then
		controls.left = true
	end
	
	if key == "right" or key == "d"  then
		controls.right = true
	end
	
end

function love.keyreleased(key, unicode)
	
	if key == "escape" then
		isFullscreen = not isFullscreen
		love.graphics.setMode( love.graphics.getWidth( ), love.graphics.getHeight( ), isFullscreen, false, 0 )
	end
	
	if key == "up" or key == "w" then
		controls.up = false
	end
	
	if key == "down" or key == "s"  then
		controls.down = false
	end
	
	if key == "left" or key == "a"  then
		controls.left = false
	end
	
	if key == "right" or key == "d"  then
		controls.right = false
	end
	
end