-- Sprite
Sprite = {image,quads = {},delay = 0,frame = 1,totalFrames = 0, paused = false,animating = false,looping = false, ctime = 0, scaleX = 1, scaleY = 1}

--images: Table of images to display
--times: Table of times to display each image
--frames: number of frames of the animation

function Sprite:new(atributes)
	atributes = atributes or {}
	setmetatable(atributes,self)
	self.__index = self
	return atributes
end

function Sprite:init(_image,_quads,_delay)
	self.image = _image
	self.quads = _quads
	self.delay = _delay
	self.totalFrames = #_quads
end

function Sprite:loop(_delegate)
	self.animating = true
	self.looping = true
	self.delegate = _delegate
end

function Sprite:play(_delegate)
	self.frame = 1
	self.animating = true
	self.looping = false
	self.delegate = _delegate
end

function Sprite:pause()
	self.paused = true
end

function Sprite:resume()
	self.paused = false
end

function Sprite:stop()
	self.animating = false
	self.looping = false
end

function Sprite:setSize(realW,realH,wantedW,wantedH)
	self.scaleX = wantedW/realW
	self.scaleY = wantedH/realH
end

function Sprite:update(dt)

	if self.animating and not self.paused then
		self.ctime = self.ctime + dt
		if self.ctime >= self.delay then
			self.ctime = 0
			self.frame = self.frame + 1
			if self.frame > self.totalFrames then
				if self.looping then 
					self.frame = 1
				else
					self.frame = self.totalFrames
					self:stop()
				end
				if self.delegate then self.delegate() end
			end
		end
	end
	
end

function Sprite:draw(x,y,r,_w,_h)
	love.graphics.drawq(self.image,self.quads[self.frame],x,y,r or 0, self.scaleX, self.scaleY, _w or 0, _h or 0)
end

function Sprite:drawRotated(x,y,r,_w,_h)
	love.graphics.drawq(self.image,self.quads[self.frame],x,y,r or 0, self.scaleX, self.scaleY, (_w*self.scaleX) or 0, (_h*self.scaleY) or 0)
end

function Sprite:drawCenter(x,y,w,h)
	love.graphics.drawq(self.image,self.quads[self.frame],x-w/2,y-w/2, 0, self.scaleX, self.scaleY)
end

