require "defines.lua"

-- Stage
Stage = {quads = {}, map = {}, boxes = {}, w = 0, h = 0, tileW = 0, tileH = 0, scaleX = 1, scaleY = 1 }

function Stage:new(atributes)
	atributes = atributes or {}
	setmetatable(atributes,self)
	self.__index = self
	return atributes
end

function Stage:init(_image,_quads,_map,_w,_h,_tileW,_tileH,_defaultLightOn,_defaultLightOff, _lightoffset, _lightSpeed)
	self.image = _image
	self.quads = _quads
	self.map = _map
	self.w = _w
	self.h = _h	
	self.tileW = _tileW
	self.tileH = _tileH
	self.defaultLight = _defaultLightOff or 0
	self.defaultLightOn = _defaultLightOn or 20
	self.defaultLightOff = _defaultLightOff or 0
	self.lightAnimationState = LIGHT_OFF
	self.lightOffset = _lightoffset or 5
	self.lightSpeed = _lightSpeed or 10
	self.lightTimer = 0
	self:prepareBoxes(0,0)
	self.color = {r = 1, g = 1, b = 1}
	self.offsetX = 1
	self.offsetY = 1
	self.defaultZoom = 1
	self.playerLight = 100
end

function Stage:turnOnLights()
	self.lightAnimationState = LIGHT_TURNING_ON
end


function Stage:turnOffLights()
	if self.lightAnimationState ~= LIGHT_OFF then
		self.lightAnimationState = LIGHT_TURNING_OFF
	end
end

function Stage:update(dt)

	if self.lightAnimationState == LIGHT_OFF or self.lightAnimationState == LIGHT_ON then return end
	
	if self.lightAnimationState == LIGHT_TURNING_ON then
	
		self.defaultLight = self.defaultLight + self.lightSpeed*dt
		
		if self.defaultLight >= self.defaultLightOn then
			self.lightAnimationState = LIGHT_ON 
			self.defaultLight = self.defaultLightOn
		end
		
	elseif self.lightAnimationState == LIGHT_TURNING_OFF then
		
		self.defaultLight = self.defaultLight - self.lightSpeed*dt
		
		if self.defaultLight <= self.defaultLightOff then
			self.lightAnimationState = LIGHT_OFF
			self.defaultLight = self.defaultLightOff
		end
		
	end
	
end

function Stage:getSafepoints()
	
	local safepoints = {}
	local posX, posY
	
	for y=1, self.h do
		for x=1, self.w do    
			if self.map[y][x] == MAP_SAFEPOINT then
				posX = ((x-1)*self.tileW)
				posY = ((y-1)*self.tileH)			
				table.insert(safepoints,{posX = posX, posY = posY})
			end
		end
	end
	
	return safepoints
	
end

function Stage:getItems()
	
	local items = {}
	local posX, posY, number
	local prenumber
	
	for y=1, self.h do
		for x=1, self.w do    
			prenumber = self.map[y][x]
			if prenumber > MAP_ITEM and prenumber <= MAP_MAX_ITEM then
				posX = ((x-1)*self.tileW)
				posY = ((y-1)*self.tileH)	
				number = prenumber - MAP_ITEM
				table.insert(items,{posX = posX, posY = posY, number = number})
			end
		end
	end
	
	return items
	
end

function Stage:getEvents()
	
	local events = {}
	local posX, posY
	
	for y=1, self.h do
		for x=1, self.w do    
			if self.map[y][x] >= EVENT_ZOOM_25 and self.map[y][x] <= EVENT_ZOOM_DEFAULT then
				posX = ((x-1)*self.tileW)
				posY = ((y-1)*self.tileH)
				table.insert(events,{posX = posX, posY = posY, number = self.map[y][x]})		
			end
		end
	end
	
	return events
	
end

function Stage:getEnemies()

	local enemies = {}
	local posX, posY
	
	for y=1, self.h do
		for x=1, self.w do    
			if self.map[y][x] >= ENEMY_TYPE_A and self.map[y][x] <= ENEMY_TYPE_F then
				posX = ((x-1)*self.tileW)
				posY = ((y-1)*self.tileH)
				table.insert(enemies,{posX = posX, posY = posY, number = self.map[y][x]})		
			end
		end
	end
	
	return enemies

end

function Stage:getPlayerPosition()
	
	local posX, posY
	
	for y=1, self.h do
		for x=1, self.w do    
			if self.map[y][x] == MAP_PLAYER then
				posX = ((x-1)*self.tileW)
				posY = ((y-1)*self.tileH)			
				return posX, posY
				end
		end
	end
	
	return 0,0
	
end

function Stage:prepareBoxes(posX,posY)

	local box
	
	for k,v in pairs(self.boxes) do self.boxes[k]=nil end
	self.boxes = {}
	
	for y=1, self.h do
		for x=1, self.w do       
			if self.map[y][x] > 0 and self.map[y][x] < 10 then --is physical object
				box = Box:new()
				box:init(((x-1)*self.tileW)+posX, ((y-1)*self.tileH)+posY, self.tileW, self.tileH)
				table.insert(self.boxes,box)
			end
		end
	end
	
end

function Stage:setQuadSize(quadW,quadH)
	self.scaleX = self.tileW/quadW
	self.scaleY = self.tileH/quadH
end

function Stage:getRealWidth()
	return self.w * self.tileW
end

function Stage:getRealHeight()
	return self.h * self.tileH
end

function Stage:draw(lights)

	local y, x = 1 , 1
	local tile = 0
	local posY, posX = 0, 0
	local _colorR = 0
	local _colorG = 0
	local _colorB = 0
	local light
	local distance
	
	for y=1, self.h do
		for x=1, self.w do      
			tile = self.map[y][x]
			if tile ~= 2 then --2 is empty
				_colorR = self.defaultLight
				_colorG = self.defaultLight
				_colorB = self.defaultLight
				if tile >= #self.quads then tile = 0 end
				posX = ((x-1)*self.tileW)
				posY = ((y-1)*self.tileH)			
				for _, light in ipairs(lights) do
					distance = light:getDistanceFromPoint(posX + self.tileW/2,posY + self.tileH/2)
					if distance <= light.lightR then
						_colorR = _colorR + ((1-(distance/light.lightR))*light.intensityR)
						_colorG = _colorG + ((1-(distance/light.lightR))*light.intensityG)
						_colorB = _colorB + ((1-(distance/light.lightR))*light.intensityB)
					end
					if _colorR > 255 then _colorR = 255 end
					if _colorG > 255 then _colorG = 255 end
					if _colorB > 255 then _colorB = 255 end
					love.graphics.setColor(_colorR*self.color.r, _colorG*self.color.g, _colorB*self.color.b,255)
				end
				
				love.graphics.drawq(self.image, self.quads[tile + 1], posX, posY, 0, self.scaleX, self.scaleY)
			end
		end
	end
	
	if SHOW_DEBUG_BOXES then
		local box
		for _, box in ipairs(self.boxes) do
			box:debugDraw()
		end
	end
	
end

