require "defines.lua"
require "stage.lua"
require "stage7.lua"
require "stage6.lua"
require "stage5.lua"
require "stage4.lua" 
require "stage3.lua" 
require "stage2.lua"
require "stage1.lua"
require "stage0.lua"
require "hero.lua"
require "camera.lua"
require "light.lua" 
require "item.lua"
require "indicator.lua" 
require "enemyA.lua"
require "enemyB.lua"

-- Stage Manager
StageManager = {stage = {}, enemies = {}, enemiesB = {}, items = {},  lights = {}, safepoints = {}, events = {}, boxes = {}}

function StageManager:new(atributes)
	atributes = atributes or {}
	setmetatable(atributes,self)
	self.__index = self
	return atributes
end

function StageManager:init(_stage,_hero,_camera)
	
	self.relaxMusic = love.audio.newSource("relax.wav")
	self.relaxMusic:setLooping(true) 
	
	self.stressMusic = love.audio.newSource("stress.mp3")
	self.stressMusic:setLooping(true) 
		
	
	self.realScreenW = love.graphics.getWidth()
	self.realScreenH = love.graphics.getHeight()
	self.screenW = self.realScreenW/2
	self.screenH = self.realScreenH/2
	self.hero = _hero
	
	self.camera = _camera
	
	self.masterlight = Light:new()
	local mX, mY = self.camera:mousePosition()
	self.masterlight:init(mX,mY,100)
	
	self.currentLevel = 0
	self.state = STATE_INTRANSITION
	
	
	self.indicator = Indicator:new()
	self.indicator:init(currentLevel, self.screenW, self.screenH)
	self.indicator.delegate = self
	
	self:loadNextLevel()
	self.camera.zoomSpeed = STAGE_TRANSITION_ZOOM_SPEED
	self.camera:scale(STAGE_TRANSITION_ZOOM)
	self.camera.delegate = self
	self.indicator.number = self.currentLevel
	self.hero.controllable = false
	self.indicator:start()
	
	self.gameSpeed = 1.0
	self.stress = false
		
end

function StageManager:loadNextLevel()

	self.gameSpeed = 1.0
	self.enemiesActive = false
	self.enemiesBActive = false
	self:createStage()
	self.stage:init()
	
	local x,y = self.stage:getPlayerPosition()
	
	self.hero:setPosition(x,y)
	self.camera:setPosition(self.hero.x + self.hero.w/2 - self.screenW/self.camera.scaleX,self.hero.y +self.hero.h/2 - self.screenH/self.camera.scaleY)
	
	
	for k,v in pairs(self.lights) do self.lights[k]=nil end
	self.lights = {}
	table.insert(self.lights,self.masterlight)
	self:initSafepoints()
	self:initItems()
	self:initEvents()
	self:initEnemies()
	self.masterlight.r = self.stage.playerLight
	self.stageLights = true
	self:stageTurnOff()
	
	--src1:setVolume(0.9) -- 90% of ordinary volume
	--self.relaxMusic:setPitch(1.0) -- one octave lower
	self.relaxMusic:setVolume(0.5)
	love.audio.play(self.relaxMusic)
	self.stressMusic:setVolume(0.5)
	self.stressMusic:setPitch(1.0)
	
end

function StageManager:createStage()
	
	if self.currentLevel == 0 then
		self.stage = Stage0:new()
	elseif self.currentLevel == 1 then
		self.stage = Stage1:new()
	elseif self.currentLevel == 2 then
		self.stage = Stage2:new()
	elseif self.currentLevel == 3 then
		self.stage = Stage3:new()
	elseif self.currentLevel == 4 then
		self.stage = Stage4:new()
	elseif self.currentLevel == 5 then
		self.stage = Stage5:new()
	elseif self.currentLevel == 6 then
		self.stage = Stage6:new()
	elseif self.currentLevel == 7 then
		self.stage = Stage7:new()
	else
		self.stage = Stage0:new()
	end
	
end

function StageManager:initItems()
	
	for k,v in pairs(self.items) do self.items[k]=nil end
	self.currentItem = 1
	self.hero.circleorbs:setNumber(self.currentItem-1)
	self.items = {}
	local _items = self.stage:getItems()
	local item
	local newItem
	local light
	
	for _, item in ipairs(_items) do
		
		newItem = Item:new()
		newItem:init(item.number, item.posX, item.posY)
		
		light = Light:new()
		light:init(item.posX + newItem.w/2, item.posY + newItem.h/2,ITEM_LIGHT_INTENSITY)
		light.intensityR = 60
		light.intensityG = 13
		light.intensityB = 55
		newItem.light = light
		table.insert(self.lights,light)
		table.insert(self.items,newItem)
    end
	
	self.totalItems = #self.items
	
end

function StageManager:initEvents()
	
	for k,v in pairs(self.events) do self.events[k]=nil end
	
	local _events = self.stage:getEvents()
	local event
	local newEvent
	
	for _, event in ipairs(_events) do
		newEvent = Box:new()
		newEvent:init(event.posX,event.posY,self.stage.tileW,self.stage.tileH)
		newEvent.event = event.number
		table.insert(self.events,newEvent)
    end
	
end

function StageManager:initEnemies()

	for k,v in pairs(self.enemies) do self.enemies[k]=nil end
	
	self.enemies = {}
	self.enemiesB = {}
	
	local _enemies = self.stage:getEnemies()
	local enemy
	local newEnemy
	local light
	local _intensity
	local _number
	local _radius
	
	for _, enemy in ipairs(_enemies) do
	
		if enemy.number >= ENEMY_TYPE_A and enemy.number < ENEMY_TYPE_B then
			newEnemy = EnemyA:new()
			_intensity = ENEMYA_LIGHT_INTENSITY
			_number = enemy.number - 9
		elseif enemy.number >= ENEMY_TYPE_B and enemy.number < ENEMY_TYPE_C then
			newEnemy = EnemyB:new()
			_intensity = ENEMYB_LIGHT_INTENSITY
			_number = enemy.number - 19
			table.insert(self.enemiesB,newEnemy)
		elseif enemy.number >= ENEMY_TYPE_C and enemy.number < ENEMY_TYPE_D then
		
		elseif enemy.number >= ENEMY_TYPE_D and enemy.number < ENEMY_TYPE_E then
		
		elseif enemy.number >= ENEMY_TYPE_E and enemy.number < ENEMY_TYPE_F then
		
		end
		
		newEnemy:init(enemy.posX,enemy.posY,_number)
		
		light = Light:new()
		light:init(newEnemy.x + newEnemy.w/2, newEnemy.y + newEnemy.h/2,_intensity)
		light.defaultLight = 0
		light.intensityR = 60
		light.intensityG = 0
		light.intensityB = 0
		light.lightAnimationState = LIGHT_OFF
		newEnemy.light = light
		
		table.insert(self.lights,light)
		table.insert(self.enemies,newEnemy)
    end
	
end

function StageManager:initSafepoints()
	
	for k,v in pairs(self.safepoints) do self.safepoints[k]=nil end
	self.safepoints = {}
	local _safepoints = self.stage:getSafepoints()
	local safepoint
	local light
	
	for _, safepoint in ipairs(_safepoints) do
		light = Light:new()
		light:init(safepoint.posX, safepoint.posY,SAFEPOINTS_LIGHT_INTENSITY)
		light.intensityR = 100
		light.intensityG = 100
		light.intensityB = 150
		table.insert(self.safepoints,light)
		table.insert(self.lights,light)
    end
	
end

function StageManager:resetCameraBounds()

	local x1 = -self.stage.tileW*self.stage.offsetX
	local y1 = -self.stage.tileH*self.stage.offsetY
	local x2 = (self.stage:getRealWidth() + self.stage.tileW*self.stage.offsetX - self.realScreenW/self.camera.scaleX)
	local y2 = (self.stage:getRealHeight() + self.stage.tileH*self.stage.offsetY - self.realScreenH/self.camera.scaleY)
	
	if x2 < 0 then x2 = 0 end
	if y2 < 0 then y2 = 0 end

	camera:setBounds(x1, y1, x2, y2)
	
end

function StageManager:update(dt,_controls)
	
	self:updateLights(dt)
	self.stage:update(dt)
	
	
	self.hero:update(dt,_controls)
	
	--Check collisions with stage
	self:updateCollisionsSolids()
	
	--Check collisions with items
	self:updateCollisionsItems(dt)
	
	--Check collisions with enemies
	self:updateCollisionsEnemies(dt)
	
	--Check collisions with events
	self:updateCollisionsEvents()
		
	if self.camera.zoomchanged then
		self:resetCameraBounds()
	end
	
	self.camera:setPosition(self.hero.x + self.hero.w/2 - self.screenW/self.camera.scaleX,self.hero.y +self.hero.h/2 - self.screenH/self.camera.scaleY)
	
	self.indicator:update(dt)
end

function StageManager:enemyBActivateEnemies()
	
	local enemy
	
	if self.enemiesActive == false then
		for _, enemy in ipairs(self.enemies) do
			enemy:activate()
			enemy.light:activate()
		end
		self.enemiesActive = true
	end
	
	if not self.stress then
		love.audio.stop(self.relaxMusic)
		love.audio.play(self.stressMusic)
		self.stress = true
	end
	
	self.enemiesBActive = true
	
end

function StageManager:activateEnemies()
	
	local enemy
	
	if self.enemiesActive == false and self.currentItem > 1 then
		for _, enemy in ipairs(self.enemies) do
			enemy:activate()
			enemy.light:activate()
		end
		self.enemiesActive = true
	end
	
	if self.enemiesBActive == false then
		for _, enemy in ipairs(self.enemiesB) do
			enemy:activate()
			enemy.light:activate()
		end
		self.enemiesBActive = true
	end
	
	
end

function StageManager:disableEnemies()
	if self.enemiesActive == true then
		for _, enemy in ipairs(self.enemies) do
			enemy:deactivate()
			enemy.light:deactivate()
		end
		self.enemiesActive = false
	end
	
	if self.enemiesBActive == true then
		for _, enemy in ipairs(self.enemiesB) do
			enemy:deactivate()
			enemy.light:deactivate()
			enemy.awake = false
		end
		self.enemiesBActive = false
		
		if self.stress and self.state == STATE_INGAME then
			love.audio.stop(self.stressMusic)
			love.audio.play(self.relaxMusic)
			self.stress = false
		end
		
	end
end

function StageManager:stageTurnOff()
	if self.stageLights then
		
		local item
		
		--turn off items
		for _, item in ipairs(self.items) do
			if self.currentItem ~= item.number then
				item:deactivate()
				item.light:deactivate()
			end
		end
	
		self.stage:turnOffLights()
		self.stageLights = false
		
		if self.currentItem > 1 and self.state == STATE_INGAME and #self.enemies > 0 and not self.stress then
			love.audio.stop(self.relaxMusic)
			love.audio.play(self.stressMusic)
			self.stress = true
		end
	end
end

function StageManager:stageTurnOn()
	if not self.stageLights then
		
		--turn on items
		for _, item in ipairs(self.items) do
			if self.currentItem ~= item.number then
				item:activate()
				item.light:activate()
			end
		end
		
		if self.currentItem > self.totalItems then
			self:gameCleared()
			self.stage.defaultLightOn = 255
			self.stage.lightSpeed = 100
		end
		
		self.stage:turnOnLights()
		self.stageLights = true
		
		if self.stress and self.state == STATE_INGAME and #self.enemies > 0 then
			love.audio.stop(self.stressMusic)
			love.audio.play(self.relaxMusic)
			self.stress = false
		end
	end
end

function StageManager:gameCleared()
	self:playClearSound()
	-- Game cleared
	love.audio.stop(self.relaxMusic)
	love.audio.stop(self.stressMusic)
	self.state = STATE_INTRANSITION
	self.currentLevel = self.currentLevel + 1
	self.camera.zoomSpeed = STAGE_TRANSITION_ZOOM_SPEED
	self.camera:zoom(STAGE_TRANSITION_ZOOM)
	self.indicator.number = self.currentLevel
	self.hero:resetAcceleration()
	self.hero.controllable = false
	self.stress = false
end

function StageManager:zoomFinished()
	if self.state == STATE_INTRANSITION then
		self.indicator:setNumberTo(self.currentLevel)
		self.indicator:start()
		
		if self.currentLevel == TOTAL_LEVELS then
			self.indicator.showThanks = true
		end
	end
end

function StageManager:indicatorFinished()
	if self.state == STATE_INTRANSITION then
		self.state = STATE_INGAME
		self:loadNextLevel()
		self.camera:zoom(self.stage.defaultZoom)
		self.hero.controllable = true
	end
end

function StageManager:gameOver()
	if self.state == STATE_INGAME then
		self.state = STATE_INTRANSITION
		--self.currentLevel = 0
		self.camera.zoomSpeed = STAGE_TRANSITION_ZOOM_SPEED
		self.camera:zoom(STAGE_TRANSITION_ZOOM)
		self.indicator.number = self.currentLevel
		self.hero:reset()
		self.hero.controllable = false
		self:playDeadSound()
		love.audio.stop(self.relaxMusic)
		love.audio.stop(self.stressMusic)
		self.stress = false
	end
end

function StageManager:updateLights(dt)

	local ignoreReset = false
	local mX, mY = self.camera:mouseRealPosition()
	self.masterlight:setPosition(mX, mY)
	
	for _, safepoint in ipairs(self.safepoints) do
		if not self.hero:goesOutFromCircle(safepoint) then
			self:stageTurnOn()
			self:disableEnemies()
			self.hero:activate()
			ignoreReset = true
			break
		end
	end
	
	if not ignoreReset then 
		if self.hero:goesOutFromCircle(self.masterlight) then
			self.hero:reset() 
			self.hero:deactivate()
		else
			self.hero:activate()
		end
		self:stageTurnOff()
		self:activateEnemies()
	end
	
	local light
	
	for _, light in ipairs(self.lights) do
		light:update(dt)
	end
	
end


function StageManager:updateCollisionsItems(dt)
	
	if self.state == STATE_INTRANSITION then return end
	
	local item
	
	for _, item in ipairs(self.items) do
		item:update(dt)
		
		if self.currentItem == item.number then
			if self.hero:collide(item) then--if self.hero:collideWithCircle(item) then --circle collision needs some debugging
				
				self.gameSpeed = self.gameSpeed + SPEED_UPDATE_PER_ITEM
				self.stressMusic:setPitch(1.0 + 0.2*(self.currentItem-1))
				
				item:die()
				item.light:die()
				self.currentItem = self.currentItem + 1
				self.hero.circleorbs:setNumber(self.currentItem-1)
				self:playItemSound(item.number)
			
				if item.number == 1 and #self.enemies > 0 and #self.enemies ~= #self.enemiesB then
					self.relaxMusic:setVolume(0.5)
					love.audio.stop(self.relaxMusic)
					love.audio.play(self.stressMusic)
					self.stress = true
				end
				break
			end
			
			
			
		end
    end
	
	
	
	for _, item in ipairs(self.items) do
		if self.currentItem == item.number then
			item:activate()
			item.light:activate()
			break
		end
    end
	
	
end


function StageManager:updateCollisionsEnemies(_dt)
	
	if self.state == STATE_INTRANSITION then return end
	
	local enemy
	local dt = _dt*self.gameSpeed
	
	for _, enemy in ipairs(self.enemies) do
		enemy:update(dt,self.hero)
		enemy.light.x = enemy.x + enemy.w/2
		enemy.light.y = enemy.y + enemy.h/2
		if self.enemiesActive and self.hero:collide(enemy) then
			if enemy.type == "B" and enemy.awake == false then break end
			self:gameOver()
			return
		end
    end
	
	for _, enemy in ipairs(self.enemiesB) do
		if enemy.lightAnimationState == LIGHT_ON and not self.hero:goesOutFromCircle(enemy.light) and self.hero.lightAnimationState == LIGHT_ON then
			self:enemyBActivateEnemies()
			enemy.awake = true
		end
    end
		
end

function StageManager:updateCollisionsEvents()
	
	if self.state == STATE_INTRANSITION then return end
	
	local event
	
	for _, event in ipairs(self.events) do

		if self.hero:collide(event) then
			
			self.camera.zoomSpeed = EVENT_ZOOM_SPEED
			
			if event.event == EVENT_ZOOM_25 then
				self.camera:zoom(.25)
			elseif event.event == EVENT_ZOOM_50 then
				self.camera:zoom(.50)
			elseif event.event == EVENT_ZOOM_75 then
				self.camera:zoom(.75)
			elseif event.event == EVENT_ZOOM_100 then
				self.camera:zoom(1)
			elseif event.event == EVENT_ZOOM_150 then
				self.camera:zoom(1.5)
			elseif event.event == EVENT_ZOOM_DEFAULT then
				self.camera:zoom(self.stage.defaultZoom)
			end
			
			break
		end

    end
	
end

function StageManager:updateCollisionsSolids()
	
	local box
	local dX
	local dY
	local checkH = true
	local checkV = true

	for _, box in ipairs(self.stage.boxes) do
		if self.hero:collide(box) then
		
			dX = math.abs((self.hero.x + self.hero.w/2) - (box.x + box.w/2))
			dY = math.abs((self.hero.y + self.hero.h/2) - (box.y + box.h/2))
			
			if dX >= dY and checkH then
				if self.hero.x >= box.x then
					self.hero.x = box.x + box.w
					if self.hero.velX < 0 then
						if USE_REBOUND then 
							self.hero.velX = -self.hero.velX 
							self:playReboteSound()
						else
							self.hero.velX = 0
						end
					end
				else
					self.hero.x = box.x - self.hero.w
					if self.hero.velX > 0 then
						if USE_REBOUND then 
							self.hero.velX = -self.hero.velX
							self:playReboteSound()
						else
							self.hero.velX = 0
						end

					end
				end
				checkH = false
			elseif dX <= dY and checkV then
			
				if self.hero.y >= box.y then
					self.hero.y = box.y + box.h
					if self.hero.velY < 0 then
						if USE_REBOUND then 
							self.hero.velY = -self.hero.velY
							self:playReboteSound()
						else
							self.hero.velY = 0
						end
					end
				else
					self.hero.y = box.y - self.hero.h
					if self.hero.velY > 0 then
						if USE_REBOUND then 
							self.hero.velY = -self.hero.velY
							self:playReboteSound()
						else
							self.hero.velY = 0
						end
					end
				end
				checkV = false
			end
			
		end
	end
	
end

function StageManager:offCameraDraw() --Draw without camera influence
	self.indicator:draw()
end



function StageManager:draw()


	self.stage:draw(self.lights)
	self.hero:draw()
		
	local item
	local enemy
		
	for _, item in ipairs(self.items) do
		item:draw()
    end
	
	for _, enemy in ipairs(self.enemies) do
		enemy:draw()
    end
	
	
	if SHOW_DEBUG_BOXES then
		self.masterlight:debugDraw()
		local event
		for _, event in ipairs(self.events) do
			event:debugDraw()
		end
		for _, light in ipairs(self.lights) do
			light:debugDraw()
		end
	end
	
	--love.graphics.setColor(255,255,255)
	--love.graphics.print(" "..mX,0,100)
	
end

-- Sounds

function StageManager:playReboteSound()
	--sounds ugly xD
	--local src = love.audio.newSource("rebote.wav", "static")
	--love.audio.play(src)
end

function StageManager:playClearSound()
	local src = love.audio.newSource("clear.wav", "static")
	love.audio.play(src)
end


function StageManager:playDeadSound()
	local src = love.audio.newSource("muerte.wav", "static")
	love.audio.play(src)
end

function StageManager:playItemSound(_number)
	
	local src
	
	if _number == 1 then
		src = love.audio.newSource("item_1.mp3", "static")
	elseif _number == 2 then
		src = love.audio.newSource("item_2.mp3", "static")
	elseif _number == 3 then
		src = love.audio.newSource("item_3.mp3", "static")
	elseif _number == 4 then
		src = love.audio.newSource("item_4.mp3", "static")
	elseif _number == 5 then
		src = love.audio.newSource("item_5.mp3", "static")
	elseif _number == 6 then
		src = love.audio.newSource("item_6.mp3", "static")
	elseif _number == 7 then
		src = love.audio.newSource("item_7.mp3", "static")
	elseif _number == 8 then
		src = love.audio.newSource("item_8.mp3", "static")
	elseif _number == 9 then
		src = love.audio.newSource("item_9.mp3", "static")
	else
		src = love.audio.newSource("item_9.mp3", "static")
	end
	
	src:setVolume(0.2)
	love.audio.play(src)
end